@@ -10,8 +10,8 @@
 
 
 import logging
-import os
 import re
+import requests
 from os.path import expanduser
 from collections import OrderedDict
 from urllib.parse import unquote as urlunquote
@@ -27,7 +27,17 @@ from datalad.interface.common_opts import (
 from datalad.log import log_progress
 from datalad.support.gitrepo import (
     GitRepo,
-    GitCommandError,
+)
+from datalad.cmd import (
+    CommandError,
+    GitRunner,
+    StdOutCapture,
+    WitlessRunner,
+)
+from datalad.distributed.ora_remote import (
+    LocalIO,
+    RIARemoteError,
+    SSHRemoteIO,
 )
 from datalad.support.annexrepo import AnnexRepo
 from datalad.support.constraints import (
@@ -48,10 +58,12 @@ from datalad.dochelpers import (
     single_or_plural,
 )
 from datalad.utils import (
-    rmtree,
     assure_bool,
     knows_annex,
+    make_tempfile,
     Path,
+    PurePosixPath,
+    rmtree,
 )
 
 from datalad.distribution.dataset import (
@@ -64,6 +76,9 @@ from datalad.distribution.dataset import (
 from datalad.distribution.utils import (
     _get_flexible_source_candidates,
 )
+from datalad.utils import (
+    check_symlink_capability
+)
 
 __docformat__ = 'restructuredtext'
 
@@ -85,12 +100,13 @@ class Clone(Interface):
     Primary differences over a direct `git clone` call are 1) the automatic
     initialization of a dataset annex (pure Git repositories are equally
     supported); 2) automatic registration of the newly obtained dataset as a
-    subdataset (submodule), if a parent dataset is specified; and 3) support
+    subdataset (submodule), if a parent dataset is specified; 3) support
     for additional resource identifiers (DataLad resource identifiers as used
-    on datasets.datalad.org, and RIA store URLs as used for store.datalad.org;
-    see examples); and 4) automatic configurable generation of alternative
-    access URL for common cases (such as appending '.git' to the URL in case
-    the accessing the base URL failed).
+    on datasets.datalad.org, and RIA store URLs as used for store.datalad.org
+    - optionally in specific versions as identified by a branch or a tag; see
+    examples); and 4) automatic configurable generation of alternative access
+    URL for common cases (such as appending '.git' to the URL in case the
+    accessing the base URL failed).
 
     || PYTHON >>By default, the command returns a single Dataset instance for
     an installed dataset, regardless of whether it was newly installed ('ok'
@@ -99,7 +115,7 @@ class Clone(Interface):
 
     .. seealso::
 
-      http://handbook.datalad.org/en/latest/usecases/datastorage_for_institutions.html
+      :ref:`handbook:3-001`
         More information on Remote Indexed Archive (RIA) stores
     """
     # by default ignore everything but install results
@@ -121,23 +137,35 @@ class Clone(Interface):
              code_cmd="datalad clone "
              "https://github.com/datalad-datasets/longnow-podcasts.git"),
         dict(text="Install a dataset into a specific directory",
-             code_py="clone("
-             "source='https://github.com/datalad-datasets/longnow"
-             "-podcasts.git', path='myfavpodcasts')",
-             code_cmd="datalad clone "
-             "https://github.com/datalad-datasets/longnow-podcasts.git "
-             "myfavpodcasts"),
+             code_py="""\
+             clone(source='https://github.com/datalad-datasets/longnow-podcasts.git',
+                   path='myfavpodcasts')""",
+             code_cmd="""\
+             datalad clone https://github.com/datalad-datasets/longnow-podcasts.git \\
+             myfavpodcasts"""),
         dict(text="Install a dataset as a subdataset into the current dataset",
-             code_py="clone(dataset='.', "
-             "source='https://github.com/datalad-datasets/longnow-podcasts.git')",
+             code_py="""\
+             clone(dataset='.',
+                   source='https://github.com/datalad-datasets/longnow-podcasts.git')""",
              code_cmd="datalad clone -d . "
              "https://github.com/datalad-datasets/longnow-podcasts.git"),
         dict(text="Install the main superdataset from datasets.datalad.org",
              code_py="clone(source='///')",
              code_cmd="datalad clone ///"),
-        dict(text="Install a dataset identified by its ID from store.datalad.org",
-             code_py="clone(source='ria+http://store.datalad.org#76b6ca66-36b1-11ea-a2e6-f0d5bf7b5561')",
-             code_cmd="datalad clone ria+http://store.datalad.org#76b6ca66-36b1-11ea-a2e6-f0d5bf7b5561"),
+        dict(text="Install a dataset identified by a literal alias from store.datalad.org",
+             code_py="clone(source='ria+http://store.datalad.org#~hcp-openaccess')",
+             code_cmd="datalad clone ria+http://store.datalad.org#~hcp-openaccess"),
+        dict(
+            text="Install a dataset in a specific version as identified by a"
+                 "branch or tag name from store.datalad.org",
+            code_py="clone(source='ria+http://store.datalad.org#76b6ca66-36b1-11ea-a2e6-f0d5bf7b5561@myidentifier')",
+            code_cmd="datalad clone ria+http://store.datalad.org#76b6ca66-36b1-11ea-a2e6-f0d5bf7b5561@myidentifier"),
+        dict(
+            text="Install a dataset with group-write access permissions",
+            code_py=\
+            "clone(source='http://example.com/dataset', reckless='shared-group')",
+            code_cmd=\
+            "datalad clone http://example.com/dataset --reckless shared-group"),
     ]
 
     _params_ = dict(
@@ -301,8 +329,8 @@ def clone_dataset(
       Any suitable clone source specifications (paths, URLs)
     destds : Dataset
       Dataset instance for the clone destination
-    reckless : {None, 'auto'}, optional
-      Mode switch to put cloned dataset into throw-away configurations, i.e.
+    reckless : {None, 'auto', 'ephemeral', 'shared-...'}, optional
+      Mode switch to put cloned dataset into unsafe/throw-away configurations, i.e.
       sacrifice data safety for performance or resource footprint.
     description : str, optional
       Location description for the annex of the dataset clone (if there is any).
@@ -410,16 +438,8 @@ def clone_dataset(
                 clone_options=clone_opts,
                 create=True)
 
-        except GitCommandError as e:
-            # Whenever progress reporting is enabled, as it is now,
-            # we end up without e.stderr since it is "processed" out by
-            # GitPython/our progress handler.
+        except CommandError as e:
             e_stderr = e.stderr
-            from datalad.support.gitrepo import GitPythonProgressBar
-            if not e_stderr and GitPythonProgressBar._last_error_lines:
-                e_stderr = os.linesep.join(GitPythonProgressBar._last_error_lines)
-                # Mimic format set in GitCommandError.__init__().
-                e.stderr = "{}  stderr: {}".format(os.linesep, e_stderr)
 
             error_msgs[cand['giturl']] = e
             lgr.debug("Failed to clone from URL: %s (%s)",
@@ -445,17 +465,13 @@ def clone_dataset(
                 )
                 yield get_status_dict(
                     status='error',
-                    message=re_match.group(1) if re_match else "stderr: " + e_stderr,
+                    message=re_match.group(1).strip()
+                    if re_match else "stderr: " + e_stderr,
                     **result_props)
                 return
             # next candidate
             continue
 
-        # perform any post-processing that needs to know details of the clone
-        # source
-        if cand['type'] == 'ria':
-            yield from postclonecfg_ria(destds, cand)
-
         result_props['source'] = cand
         # do not bother with other sources if succeeded
         break
@@ -495,47 +511,214 @@ def clone_dataset(
             **result_props)
         return
 
+    if not cand.get("version"):
+        postclone_check_head(destds)
+
+    # act on --reckless=shared-...
+    # must happen prior git-annex-init, where we can cheaply alter the repo
+    # setup through safe re-init'ing
+    if reckless and reckless.startswith('shared-'):
+        lgr.debug('Reinit %s to enable shared access permissions', destds)
+        destds.repo.call_git(['init', '--shared={}'.format(reckless[7:])])
+
     yield from postclonecfg_annexdataset(
         destds,
         reckless,
         description)
 
+    # perform any post-processing that needs to know details of the clone
+    # source
+    if result_props['source']['type'] == 'ria':
+        yield from postclonecfg_ria(destds, result_props['source'])
+
     # yield successful clone of the base dataset now, as any possible
     # subdataset clone down below will not alter the Git-state of the
     # parent
     yield get_status_dict(status='ok', **result_props)
 
 
+def postclone_check_head(ds):
+    repo = ds.repo
+    if not repo.commit_exists("HEAD"):
+        # HEAD points to an unborn branch. A likely cause of this is that the
+        # remote's main branch is something other than master but HEAD wasn't
+        # adjusted accordingly.
+        #
+        # Let's choose the most recently updated remote ref (according to
+        # commit date). In the case of a submodule, switching to a ref with
+        # commits prevents .update_submodule() from failing. It is likely that
+        # the ref includes the registered commit, but we don't have the
+        # information here to know for sure. If it doesn't, .update_submodule()
+        # will check out a detached HEAD.
+        remote_branches = (
+            b["refname:strip=2"] for b in repo.for_each_ref_(
+                fields="refname:strip=2", sort="-committerdate",
+                pattern="refs/remotes/origin"))
+        for rbranch in remote_branches:
+            if rbranch in ["origin/git-annex", "HEAD"]:
+                continue
+            repo.call_git(["checkout", "-b", rbranch[7:],  # drop "origin/"
+                           "--track", rbranch])
+            lgr.debug("Checked out local branch from %s", rbranch)
+            return
+        lgr.warning("Cloned %s but could not find a branch "
+                    "with commits", ds.path)
+
+
 def postclonecfg_ria(ds, props):
     """Configure a dataset freshly cloned from a RIA store"""
+    repo = ds.repo
     # RIA uses hashdir mixed, copying data to it via git-annex (if cloned via
-    # ssh) would make it see a bare repo and establish a hashdir lower annex object
-    # tree.
-    # Moreover, we want the RIA remote to receive all data for the store, so its
+    # ssh) would make it see a bare repo and establish a hashdir lower annex
+    # object tree.
+    # Moreover, we want the ORA remote to receive all data for the store, so its
     # objects could be moved into archives (the main point of a RIA store).
+    RIA_REMOTE_NAME = 'origin'  # don't hardcode everywhere
     ds.config.set(
-        'remote.origin.annex-ignore', 'true',
+        'remote.{}.annex-ignore'.format(RIA_REMOTE_NAME), 'true',
         where='local')
 
-    # chances are that if this dataset came from a RIA store, its subdatasets may live
-    # there too. Place a subdataset source candidate config that makes get probe this
-    # RIA store when obtaining subdatasets
+    # chances are that if this dataset came from a RIA store, its subdatasets
+    # may live there too. Place a subdataset source candidate config that makes
+    # get probe this RIA store when obtaining subdatasets
     ds.config.set(
         # we use the label 'origin' for this candidate in order to not have to
-        # generate a complicated name from the actual source specification
-        'datalad.get.subdataset-source-candidate-origin',
+        # generate a complicated name from the actual source specification.
+        # we pick a cost of 200 to sort it before datalad's default candidates
+        # for non-RIA URLs, because they prioritize hierarchical layouts that
+        # cannot be found in a RIA store
+        'datalad.get.subdataset-source-candidate-200origin',
         # use the entire original URL, up to the fragment + plus dataset ID
         # placeholder, this should make things work with any store setup we
         # support (paths, ports, ...)
         props['source'].split('#', maxsplit=1)[0] + '#{id}',
         where='local')
 
-    # TODO setup publication dependency, if a corresponding special remote exists
+    # setup publication dependency, if a corresponding special remote exists
     # and was enabled (there could be RIA stores that actually only have repos)
-    # make this function be a generator even though it doesn't actually yield
-    # anything yet
-    if None:
-        yield None
+    # make this function be a generator
+    ora_remotes = [s for s in ds.siblings('query', result_renderer='disabled')
+                   if s.get('annex-externaltype') == 'ora']
+    if not ora_remotes and any(
+            r.get('externaltype') == 'ora'
+            for r in (repo.get_special_remotes().values()
+                      if hasattr(repo, 'get_special_remotes')
+                      else [])):
+        # no ORA remote autoenabled, but configuration known about at least one.
+        # Let's check origin's config for datalad.ora-remote.uuid as stored by
+        # create-sibling-ria and enable try enabling that one.
+        lgr.debug("Found no autoenabled ORA special remote. Trying to look it "
+                  "up in source config ...")
+
+        # First figure whether we cloned via SSH, HTTP or local path and then
+        # get that config file the same way:
+        config_content = None
+        scheme = props['giturl'].split(':', 1)[0]
+        if scheme == 'http':
+
+            try:
+                response = requests.get("{}{}config".format(
+                    props['giturl'],
+                    '/' if not props['giturl'].endswith('/') else '')
+                )
+                config_content = response.text
+            except requests.RequestException as e:
+                lgr.debug("Failed to get config file from source:\n%s",
+                          exc_str(e))
+
+        elif scheme == 'ssh':
+            # TODO: switch the following to proper command abstraction:
+            # SSHRemoteIO ignores the path part ATM. No remote CWD! (To be
+            # changed with command abstractions). So we need to get that part to
+            # have a valid path to origin's config file:
+            cfg_path = PurePosixPath(URL(props['giturl']).path) / 'config'
+            op = SSHRemoteIO(props['giturl'])
+            try:
+                config_content = op.read_file(cfg_path)
+            except RIARemoteError as e:
+                lgr.debug("Failed to get config file from source: %s",
+                          exc_str(e))
+
+        elif scheme == 'file':
+            # TODO: switch the following to proper command abstraction:
+            op = LocalIO()
+            cfg_path = Path(URL(props['giturl']).localpath) / 'config'
+            try:
+                config_content = op.read_file(cfg_path)
+            except (RIARemoteError, OSError) as e:
+                lgr.debug("Failed to get config file from source: %s",
+                          exc_str(e))
+        else:
+            lgr.debug("Unknown URL-Scheme in %s. Can handle SSH, HTTP or "
+                      "FILE scheme URLs.", props['source'])
+
+        # 3. And read it
+        org_uuid = None
+        if config_content:
+            # TODO: We might be able to spare the saving to a file.
+            #       "git config -f -" is not explicitly documented but happens
+            #       to work and would read from stdin. Make sure we know this
+            #       works for required git versions and on all platforms.
+            with make_tempfile(content=config_content) as cfg_file:
+                runner = WitlessRunner(env=GitRunner.get_git_environ_adjusted())
+                try:
+                    result = runner.run(
+                        ['git', 'config', '-f', cfg_file,
+                         'datalad.ora-remote.uuid'],
+                        protocol=StdOutCapture
+                    )
+                    org_uuid = result['stdout'].strip()
+                except CommandError as e:
+                    # doesn't contain what we are looking for
+                    lgr.debug("Found no UUID for ORA special remote at "
+                              "'%s' (%s)", RIA_REMOTE_NAME, exc_str(e))
+
+        # Now, enable it. If annex-init didn't fail to enable it as stored, we
+        # wouldn't end up here, so enable with store URL as suggested by the URL
+        # we cloned from.
+        if org_uuid:
+            srs = repo.get_special_remotes()
+            if org_uuid in srs.keys():
+                # TODO: - Double-check autoenable value and only do this when
+                #         true?
+                #       - What if still fails? -> Annex shouldn't change config
+                #         in that case
+
+                # we only need the store:
+                new_url = props['source'].split('#')[0]
+                try:
+                    repo.enable_remote(srs[org_uuid]['name'],
+                                       options=['url={}'.format(new_url)]
+                                       )
+                    lgr.info("Reconfigured %s for %s",
+                             srs[org_uuid]['name'], new_url)
+                    # update ora_remotes for considering publication dependency
+                    # below
+                    ora_remotes = [s for s in
+                                   ds.siblings('query',
+                                               result_renderer='disabled')
+                                   if s.get('annex-externaltype', None) ==
+                                   'ora']
+                except CommandError as e:
+                    lgr.debug("Failed to reconfigure ORA special remote: %s",
+                              exc_str(e))
+            else:
+                lgr.debug("Unknown ORA special remote uuid at '%s': %s",
+                          RIA_REMOTE_NAME, org_uuid)
+    if ora_remotes:
+        if len(ora_remotes) == 1:
+            yield from ds.siblings('configure',
+                                   name=RIA_REMOTE_NAME,
+                                   publish_depends=ora_remotes[0]['name'],
+                                   result_filter=None,
+                                   result_renderer='disabled')
+        else:
+            lgr.warning("Found multiple ORA remotes. Couldn't decide which "
+                        "publishing to 'origin' should depend on: %s. Consider "
+                        "running 'datalad siblings configure -s origin "
+                        "--publish-depends ORAREMOTENAME' to set publication "
+                        "dependency manually.",
+                        [r['name'] for r in ora_remotes])
 
 
 def postclonecfg_annexdataset(ds, reckless, description=None):
@@ -550,37 +733,91 @@ def postclonecfg_annexdataset(ds, reckless, description=None):
         return
 
     # init annex when traces of a remote annex can be detected
-    if reckless:
+    if reckless == 'auto':
         lgr.debug(
             "Instruct annex to hardlink content in %s from local "
             "sources, if possible (reckless)", ds.path)
-        # store the reckless setting in the dataset to make it
-        # known to later clones of subdatasets via get()
-        ds.config.set(
-            'datalad.clone.reckless', reckless,
-            where='local',
-            # delay reload until all config IO is done
-            reload=False)
         ds.config.set(
             'annex.hardlink', 'true', where='local', reload=True)
 
-    # we have just cloned the repo, so it has 'origin', configure any
-    # reachable origin of origins
-    yield from configure_origins(ds, ds)
-
     lgr.debug("Initializing annex repo at %s", ds.path)
     # Note, that we cannot enforce annex-init via AnnexRepo().
     # If such an instance already exists, its __init__ will not be executed.
-    # Therefore do quick test once we have an object and decide whether to call its _init().
+    # Therefore do quick test once we have an object and decide whether to call
+    # its _init().
     #
     # Additionally, call init if we need to add a description (see #1403),
     # since AnnexRepo.__init__ can only do it with create=True
     repo = AnnexRepo(ds.path, init=True)
     if not repo.is_initialized() or description:
         repo._init(description=description)
-    if reckless:
+    if reckless == 'auto' or (reckless and reckless.startswith('shared-')):
         repo._run_annex_command('untrust', annex_options=['here'])
 
+    elif reckless == 'ephemeral':
+        # with ephemeral we declare 'here' as 'dead' right away, whenever
+        # we symlink origin's annex, since availability from 'here' should
+        # not be propagated for an ephemeral clone when we publish back to
+        # origin.
+        # This will cause stuff like this for a locally present annexed file:
+        # % git annex whereis d1
+        # whereis d1 (0 copies) failed
+        # BUT this works:
+        # % git annex find . --not --in here
+        # % git annex find . --in here
+        # d1
+
+        # we don't want annex copy-to origin
+        ds.config.set(
+            'remote.origin.annex-ignore', 'true',
+            where='local')
+
+        ds.repo.set_remote_dead('here')
+
+        if check_symlink_capability(ds.repo.dot_git / 'dl_link_test',
+                                    ds.repo.dot_git / 'dl_target_test'):
+            # symlink the annex to avoid needless copies in an emphemeral clone
+            annex_dir = ds.repo.dot_git / 'annex'
+            origin_annex_url = ds.config.get("remote.origin.url", None)
+            origin_git_path = None
+            if origin_annex_url:
+                try:
+                    # Deal with file:// scheme URLs as well as plain paths.
+                    # If origin isn't local, we have nothing to do.
+                    origin_git_path = Path(RI(origin_annex_url).localpath)
+                    if origin_git_path.name != '.git':
+                        origin_git_path /= '.git'
+                except ValueError:
+                    # Note, that accessing localpath on a non-local RI throws
+                    # ValueError rather than resulting in an AttributeError.
+                    # TODO: Warning level okay or is info level sufficient?
+                    # Note, that setting annex-dead is independent of
+                    # symlinking .git/annex. It might still make sense to
+                    # have an ephemeral clone that doesn't propagate its avail.
+                    # info. Therefore don't fail altogether.
+                    lgr.warning("reckless=ephemeral mode: origin doesn't seem "
+                                "local: %s\nno symlinks being used",
+                                origin_annex_url)
+            if origin_git_path:
+                # TODO make sure that we do not delete any unique data
+                rmtree(str(annex_dir)) \
+                    if not annex_dir.is_symlink() else annex_dir.unlink()
+                annex_dir.symlink_to(origin_git_path / 'annex',
+                                     target_is_directory=True)
+        else:
+            # TODO: What level? + note, that annex-dead is independ
+            lgr.warning("reckless=ephemeral mode: Unable to create symlinks on "
+                        "this file system.")
+
+    if reckless:
+        # we successfully dealt with reckless here.
+        # store the reckless setting in the dataset to make it
+        # known to later clones of subdatasets via get()
+        ds.config.set(
+            'datalad.clone.reckless', reckless,
+            where='local',
+            reload=True)
+
     srs = {True: [], False: []}  # special remotes by "autoenable" key
     remote_uuids = None  # might be necessary to discover known UUIDs
 
@@ -600,8 +837,9 @@ def postclonecfg_annexdataset(ds, reckless, description=None):
         except ValueError:
             lgr.warning(
                 'Failed to process "autoenable" value %r for sibling %s in '
-                'dataset %s as bool.  You might need to enable it later '
-                'manually and/or fix it up to avoid this message in the future.',
+                'dataset %s as bool.'
+                'You might need to enable it later manually and/or fix it up to'
+                ' avoid this message in the future.',
                 sr_autoenable, sr_name, ds.path)
             continue
 
@@ -626,7 +864,8 @@ def postclonecfg_annexdataset(ds, reckless, description=None):
             " but no UUIDs for them yet known for dataset %s",
             # since we are only at debug level, we could call things their
             # proper names
-            single_or_plural("special remote", "special remotes", len(srs[True]), True),
+            single_or_plural("special remote",
+                             "special remotes", len(srs[True]), True),
             ", ".join(srs[True]),
             ds.path
         )
@@ -634,15 +873,22 @@ def postclonecfg_annexdataset(ds, reckless, description=None):
     if srs[False]:
         # if has no auto-enable special remotes
         lgr.info(
-            'access to %s %s not auto-enabled, enable with:\n\t\tdatalad siblings -d "%s" enable -s %s',
+            'access to %s %s not auto-enabled, enable with:\n'
+            '\t\tdatalad siblings -d "%s" enable -s %s',
             # but since humans might read it, we better confuse them with our
             # own terms!
-            single_or_plural("dataset sibling", "dataset siblings", len(srs[False]), True),
+            single_or_plural("dataset sibling",
+                             "dataset siblings", len(srs[False]), True),
             ", ".join(srs[False]),
             ds.path,
             srs[False][0] if len(srs[False]) == 1 else "SIBLING",
         )
 
+    # we have just cloned the repo, so it has 'origin', configure any
+    # reachable origin of origins
+    yield from configure_origins(ds, ds)
+
+
 _handle_possible_annex_dataset = postclonecfg_annexdataset
 
 
@@ -654,38 +900,58 @@ def configure_origins(cfgds, probeds, label=None):
     cfgds : Dataset
       Dataset to receive the remote configurations
     probeds : Dataset
-      Dataset to start looking for 'origin' remotes. May be identical with `cfgds`.
+      Dataset to start looking for 'origin' remotes. May be identical with
+      `cfgds`.
     label : int, optional
       Each discovered 'origin' will be configured as a remote under the name
-      'origin-<label>'. If no label is given, '2' will be used by default, given that
-      there is typically a 'origin' remote already.
+      'origin-<label>'. If no label is given, '2' will be used by default,
+      given that there is typically a 'origin' remote already.
     """
     if label is None:
-        label = 2
+        label = 1
     # let's look at the URL for that remote and see if it is a local
     # dataset
     origin_url = probeds.config.get('remote.origin.url')
-    if origin_url and cfgds.config.obtain(
+    if not origin_url:
+        # no origin, nothing to do
+        return
+    if not cfgds.config.obtain(
             'datalad.install.inherit-local-origin',
-            default=True) and isinstance(RI(origin_url), PathRI):
-        # given the clone source is a local dataset, we can have a
-        # cheap look at it, and configure its own 'origin' as a remote
-        # (if there is any), and benefit from additional annex availability
-        originorigin_ds = Dataset(probeds.pathobj / origin_url)
-        originorigin_url = originorigin_ds.config.get('remote.origin.url')
-        if originorigin_url:
+            default=True):
+        # no inheritance wanted
+        return
+    if not isinstance(RI(origin_url), PathRI):
+        # not local path
+        return
+
+    # no need to reconfigure original/direct origin again
+    if cfgds != probeds:
+        # prevent duplicates
+        known_remote_urls = set(
+            cfgds.config.get(r + '.url', None)
+            for r in cfgds.config.sections()
+            if r.startswith('remote.')
+        )
+        if origin_url not in known_remote_urls:
             yield from cfgds.siblings(
                 'configure',
-                # no chance for config, can only be the second configured remote
+                # no chance for conflict, can only be the second configured
+                # remote
                 name='origin-{}'.format(label),
-                url=originorigin_url,
+                url=origin_url,
                 # fetch to get all annex info
                 fetch=True,
                 result_renderer='disabled',
                 on_failure='ignore',
             )
-        # and dive deeper
-        yield from configure_origins(cfgds, originorigin_ds, label=label + 1)
+    # and dive deeper
+    # given the clone source is a local dataset, we can have a
+    # cheap look at it, and configure its own 'origin' as a remote
+    # (if there is any), and benefit from additional annex availability
+    yield from configure_origins(
+        cfgds,
+        Dataset(probeds.pathobj / origin_url),
+        label=label + 1)
 
 
 def _get_tracking_source(ds):
@@ -762,6 +1028,13 @@ def decode_source_spec(spec, cfg=None):
         source=spec,
         version=None,
     )
+
+    # Git never gets to see these URLs, so let's manually apply any
+    # rewrite configuration Git might know about.
+    # Note: We need to rewrite before parsing, otherwise parsing might go wrong.
+    # This is particularly true for insteadOf labels replacing even the URL
+    # scheme.
+    spec = cfg.rewrite_url(spec)
     # common starting point is a RI instance, support for accepting an RI
     # instance is kept for backward-compatibility reasons
     source_ri = RI(spec) if not isinstance(spec, RI) else spec
@@ -772,15 +1045,20 @@ def decode_source_spec(spec, cfg=None):
         props['type'] = 'dataladri'
         props['giturl'] = source_ri.as_git_url()
     elif isinstance(source_ri, URL) and source_ri.scheme.startswith('ria+'):
-        # Git never gets to see these URLs, so let's manually apply any
-        # rewrite configuration Git might know about
-        source_ri = RI(cfg.rewrite_url(spec))
         # parse a RIA URI
         dsid, version = source_ri.fragment.split('@', maxsplit=1) \
             if '@' in source_ri.fragment else (source_ri.fragment, None)
         uuid_regex = r'[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}'
-        if not re.match(uuid_regex, dsid):
-            raise ValueError('RIA URI does not contain a valid dataset ID: {}'.format(spec))
+        if re.match(uuid_regex, dsid):
+            trace = '{}/{}'.format(dsid[:3], dsid[3:])
+            default_destpath = dsid
+        elif dsid.startswith('~'):
+            trace = 'alias/{}'.format(dsid[1:])
+            default_destpath = dsid[1:]
+        else:
+            raise ValueError(
+                'RIA URI not recognized, no valid dataset ID or other supported '
+                'scheme: {}'.format(spec))
         # now we cancel the fragment in the original URL, but keep everthing else
         # in order to be able to support the various combinations of ports, paths,
         # and everything else
@@ -791,13 +1069,13 @@ def decode_source_spec(spec, cfg=None):
         source_ri.path = '{urlpath}{urldelim}{trace}'.format(
             urlpath=source_ri.path if source_ri.path else '',
             urldelim='' if not source_ri.path or source_ri.path.endswith('/') else '/',
-            trace='{}/{}'.format(dsid[:3], dsid[3:]),
+            trace=trace,
         )
         props.update(
             type='ria',
             giturl=str(source_ri),
             version=version,
-            default_destpath=dsid,
+            default_destpath=default_destpath,
         )
     else:
         # let's assume that anything else is a URI that Git can handle
