@@ -11,7 +11,8 @@
 
 """
 
-import os, os.path as op
+import os
+import os.path as op
 import shutil
 import sys
 import logging
@@ -19,87 +20,104 @@ from unittest.mock import patch
 import builtins
 
 from operator import itemgetter
-from os.path import dirname, normpath, pardir, basename
-from os.path import isabs, expandvars, expanduser
+from os.path import (
+    abspath,
+    basename,
+    dirname,
+    exists,
+    expanduser,
+    expandvars,
+    isabs,
+    join as opj,
+    normpath,
+    pardir,
+)
 from collections import OrderedDict
 
-from ..utils import updated
-from os.path import join as opj, abspath, exists
-from ..utils import (
-    rotree, swallow_outputs, swallow_logs, setup_exceptionhook, md5sum
+from datalad.utils import (
+    _path_,
+    all_same,
+    any_re_search,
+    assure_unicode,
+    auto_repr,
+    better_wraps,
+    CMD_MAX_ARG,
+    create_tree,
+    disable_logger,
+    dlabspath,
+    expandpath,
+    file_basename,
+    find_files,
+    generate_chunks,
+    get_dataset_root,
+    get_func_kwargs_doc,
+    get_open_files,
+    get_path_prefix,
+    get_timestamp_suffix,
+    get_trace,
+    getpwd, chpwd,
+    import_module_from_file,
+    import_modules,
+    is_explicit_path,
+    knows_annex,
+    line_profile,
+    make_tempfile,
+    map_items,
+    md5sum,
+    never_fail,
+    not_supported_on_windows,
+    on_windows,
+    partition,
+    Path,
+    path_is_subpath,
+    path_startswith,
+    rotree,
+    safe_print,
+    setup_exceptionhook,
+    swallow_logs,
+    swallow_outputs,
+    unique,
+    unlink,
+    updated,
 )
-from ..utils import getpwd, chpwd
-from ..utils import get_path_prefix
-from ..utils import auto_repr
-from ..utils import find_files
-from ..utils import is_interactive
-from ..utils import line_profile
-from ..utils import not_supported_on_windows
-from ..utils import file_basename
-from ..utils import expandpath, is_explicit_path
-from ..utils import assure_unicode
-from ..utils import knows_annex
-from ..utils import any_re_search
-from ..utils import unique
-from ..utils import all_same
-from ..utils import partition
-from ..utils import get_func_kwargs_doc
-from ..utils import make_tempfile
-from ..utils import on_windows
-from ..utils import _path_
-from ..utils import get_timestamp_suffix
-from ..utils import get_trace
-from ..utils import get_dataset_root
-from ..utils import better_wraps
-from ..utils import path_startswith
-from ..utils import path_is_subpath
-from ..utils import dlabspath
-from ..utils import safe_print
-from ..utils import generate_chunks
-from ..utils import disable_logger
-from ..utils import import_modules, import_module_from_file
-from ..utils import get_open_files
-from ..utils import map_items
-from ..utils import unlink
-from ..utils import CMD_MAX_ARG
-from ..utils import create_tree
-from ..utils import never_fail
-from ..utils import Path
+from datalad.support.annexrepo import AnnexRepo
 
-from ..support.annexrepo import AnnexRepo
-
-from nose.tools import (
+from .utils import (
+    as_unicode,
+    assert_cwd_unchanged,
     assert_equal,
     assert_false,
     assert_greater,
+    assert_in,
+    assert_not_in,
+    assert_raises,
     assert_true,
+    assure_bool,
+    assure_dict_from_str,
+    assure_iter,
+    assure_list,
+    assure_list_from_str,
+    assure_unicode,
     eq_,
-    ok_,
-)
-from datalad.tests.utils import nok_, assert_re_in
-
-from .utils import with_tempfile, assert_in, with_tree
-from .utils import SkipTest
-from .utils import assert_cwd_unchanged, skip_if_on_windows
-from .utils import assure_dict_from_str, assure_list_from_str
-from .utils import assure_unicode
-from .utils import as_unicode
-from .utils import assure_bool
-from .utils import assure_iter
-from .utils import assure_list
-from .utils import ok_generator
-from .utils import assert_not_in
-from .utils import assert_raises
-from .utils import ok_startswith
-from .utils import skip_if_no_module
-from .utils import (
-    probe_known_failure, skip_known_failure, known_failure, known_failure_v6,
-    skip_if,
-    ok_file_has_content,
-    known_failure_windows,
     has_symlink_capability,
+    known_failure,
+    known_failure_v6,
+    known_failure_windows,
+    nok_,
+    OBSCURE_FILENAME,
+    ok_,
+    ok_file_has_content,
+    ok_generator,
+    ok_startswith,
+    probe_known_failure,
+    skip_if,
+    skip_if_no_module,
+    skip_if_on_windows,
+    skip_known_failure,
+    SkipTest,
+    with_tempfile,
+    with_tree,
 )
-from .utils import OBSCURE_FILENAME
 
 
 def test_get_func_kwargs_doc():
@@ -359,9 +377,9 @@ def test_getpwd_change_mode(tdir):
     # The evil plain chdir call
     os.chdir(tdir)
     # Just testing the logic of switching to cwd mode and issuing a warning
-    with swallow_logs(new_level=logging.WARNING) as cml:
+    with swallow_logs(new_level=logging.DEBUG) as cml:
         pwd = getpwd()
-        eq_(pwd, os.path.realpath(pwd))  # might have symlinks, thus realpath
+        eq_(pwd, str(Path(pwd).resolve()))  # might have symlinks, thus realpath
     assert_in("symlinks in the paths will be resolved", cml.out)
     eq_(utils._pwd_mode, 'cwd')
 
@@ -682,27 +700,23 @@ def test_path_():
         eq_(_path_(p, 'd'), 'a/b/c/d')
 
 
-@known_failure_windows
 def test_get_timestamp_suffix():
     # we need to patch temporarily TZ
-    import time
-    try:
-        with patch.dict('os.environ', {'TZ': 'GMT'}):
-            time.tzset()
-            # skynet DOB
-            assert_equal(get_timestamp_suffix(0), '-1970-01-01T00:00:00+0000')
-            assert_equal(get_timestamp_suffix(0, prefix="+"),
-                         '+1970-01-01T00:00:00+0000')
-            # yoh found no way to mock things out and didn't want to provide
-            # explicit call to anything to get current time with the timezone,
-            # so disabling this test for now besides that it should return smth
-            # sensible ;)
-            #with patch.object(time, 'localtime', lambda: 1):
-            #    assert_equal(get_timestamp_suffix(),
-            #  '-1970-01-01T00:00:01+0000')  # skynet is 1 sec old
-            assert(get_timestamp_suffix().startswith('-'))
-    finally:
-        time.tzset()
+    with patch.dict('os.environ', {'TZ': 'GMT'}):
+        # skynet DOB
+        target_ts = '1970-01-01T00:00:00-0000' \
+            if on_windows else '1970-01-01T00:00:00+0000'
+        assert_equal(get_timestamp_suffix(0), '-' + target_ts)
+        assert_equal(get_timestamp_suffix(0, prefix="+"),
+                     '+' + target_ts)
+        # yoh found no way to mock things out and didn't want to provide
+        # explicit call to anything to get current time with the timezone,
+        # so disabling this test for now besides that it should return smth
+        # sensible ;)
+        #with patch.object(time, 'localtime', lambda: 1):
+        #    assert_equal(get_timestamp_suffix(),
+        #  '-1970-01-01T00:00:01+0000')  # skynet is 1 sec old
+        assert(get_timestamp_suffix().startswith('-'))
 
 
 def test_memoized_generator():
@@ -1160,18 +1174,29 @@ def test_dlabspath(path):
 
 @with_tree({'1': 'content', 'd': {'2': 'more'}})
 def test_get_open_files(p):
+    pobj = Path(p)
     skip_if_no_module('psutil')
     eq_(get_open_files(p), {})
-    f1 = opj(p, '1')
-    subd = opj(p, 'd')
-    with open(f1) as f:
+    f1 = pobj / '1'
+    subd = pobj / 'd'
+    with f1.open() as f:
         # since lsof does not care about PWD env var etc, paths
         # will not contain symlinks, we better realpath them
         # all before comparison
-        eq_(get_open_files(p, log_open=40)[op.realpath(f1)].pid,
+        eq_(get_open_files(p, log_open=40)[str(f1.resolve())].pid,
             os.getpid())
 
-    assert not get_open_files(subd)
+    assert not get_open_files(str(subd))
+
+    if on_windows:
+        # the remainder of the test assume a certain performance.
+        # however, on windows get_open_files() can be very slow
+        # (e.g. the first invocation in this test (above) can easily
+        # take 30-50s). It is not worth slowing the tests to
+        # accomodate this issue, given we have tested proper functioning
+        # in principle already above).
+        return
+
     # if we start a process within that directory, should get informed
     from subprocess import Popen, PIPE
     from time import time
@@ -1180,14 +1205,14 @@ def test_get_open_files(p):
                   r'import sys; sys.stdout.write("OK\n"); sys.stdout.flush();'
                   r'import time; time.sleep(10)'],
                  stdout=PIPE,
-                 cwd=subd)
+                 cwd=str(subd))
     # Assure that it started and we read the OK
     eq_(assure_unicode(proc.stdout.readline().strip()), u"OK")
     assert time() - t0 < 5 # that we were not stuck waiting for process to finish
-    eq_(get_open_files(p)[op.realpath(subd)].pid, proc.pid)
-    eq_(get_open_files(subd)[op.realpath(subd)].pid, proc.pid)
+    eq_(get_open_files(p)[str(subd.resolve())].pid, proc.pid)
+    eq_(get_open_files(subd)[str(subd.resolve())].pid, proc.pid)
     proc.terminate()
-    assert not get_open_files(subd)
+    assert not get_open_files(str(subd))
 
 
 def test_map_items():
