@@ -10,9 +10,14 @@
 import collections
 import hashlib
 import re
-import six.moves.builtins as __builtin__
+import builtins
 import time
 
+try:
+    from collections.abc import Callable
+except ImportError:  # Python <= 3.3
+    from collections import Callable
+
 import logging
 import shutil
 import os
@@ -22,39 +27,39 @@ import platform
 import gc
 import glob
 import gzip
+import stat
 import string
+import warnings
 import wrapt
 
 from copy import copy as shallow_copy
 from contextlib import contextmanager
 from functools import wraps
 from time import sleep
-from inspect import getargspec
+import inspect
 from itertools import tee
 
 import os.path as op
 from os.path import sep as dirsep
 from os.path import commonprefix
-from os.path import curdir, basename, exists, realpath, islink, join as opj
+from os.path import curdir, basename, exists, islink, join as opj
 from os.path import isabs, normpath, expandvars, expanduser, abspath, sep
 from os.path import isdir
 from os.path import relpath
-from os.path import stat
 from os.path import dirname
 from os.path import split as psplit
 import posixpath
 
-
-from six import PY2, text_type, binary_type, string_types
+from shlex import (
+    quote as shlex_quote,
+    split as shlex_split,
+)
 
 # from datalad.dochelpers import get_docstring_split
 from datalad.consts import TIMESTAMP_FMT
 
 
-if PY2:
-    unicode_srctypes = string_types
-else:
-    unicode_srctypes = string_types + (bytes,)
+unicode_srctypes = str, bytes
 
 
 lgr = logging.getLogger("datalad.utils")
@@ -70,9 +75,25 @@ on_linux = platform_system == 'linux'
 on_msys_tainted_paths = on_windows \
                         and 'MSYS_NO_PATHCONV' not in os.environ \
                         and os.environ.get('MSYSTEM', '')[:4] in ('MSYS', 'MING')
+
+
+def get_linux_distribution():
+    """Compatibility wrapper for {platform,distro}.linux_distribution().
+    """
+    if hasattr(platform, "linux_distribution"):
+        # Use deprecated (but faster) method if it's available.
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", category=DeprecationWarning)
+            result = platform.linux_distribution()
+    else:
+        import distro  # We require this for Python 3.8 and above.
+        result = distro.linux_distribution(full_distribution_name=False)
+    return result
+
+
 try:
     linux_distribution_name, linux_distribution_release \
-        = platform.linux_distribution()[:2]
+        = get_linux_distribution()[:2]
     on_debian_wheezy = on_linux \
                        and linux_distribution_name == 'debian' \
                        and linux_distribution_release.startswith('7.')
@@ -118,6 +139,16 @@ lgr.debug(
 # Little helpers
 #
 
+# `getargspec` has been deprecated in Python 3.
+if hasattr(inspect, "getfullargspec"):
+    ArgSpecFake = collections.namedtuple(
+        "ArgSpecFake", ["args", "varargs", "keywords", "defaults"])
+
+    def getargspec(func):
+        return ArgSpecFake(*inspect.getfullargspec(func)[:4])
+else:
+    getargspec = inspect.getargspec
+
 
 def get_func_kwargs_doc(func):
     """ Provides args for a function
@@ -140,7 +171,7 @@ def get_func_kwargs_doc(func):
 
 def any_re_search(regexes, value):
     """Return if any of regexes (list or str) searches succesfully for value"""
-    for regex in assure_tuple_or_list(regexes):
+    for regex in ensure_tuple_or_list(regexes):
         if re.search(regex, value):
             return True
     return False
@@ -160,7 +191,11 @@ def shortened_repr(value, l=30):
         if hasattr(value, '__repr__') and (value.__repr__ is not object.__repr__):
             value_repr = repr(value)
             if not value_repr.startswith('<') and len(value_repr) > l:
-                value_repr = "<<%s...>>" % (value_repr[:l - 8])
+                value_repr = "<<%s++%d chars++%s>>" % (
+                    value_repr[:l - 16],
+                    len(value_repr) - (l - 16 + 4),
+                    value_repr[-4:]
+                )
             elif value_repr.startswith('<') and value_repr.endswith('>') and ' object at 0x':
                 raise ValueError("I hate those useless long reprs")
         else:
@@ -252,8 +287,11 @@ def sorted_files(dout):
                        for r, d, files in os.walk(dout)
                        if not '.git' in r], []))
 
-_VCS_REGEX = r'%s\.(?:git|gitattributes|svn|bzr|hg)(?:%s|$)' % (dirsep, dirsep)
-_DATALAD_REGEX = r'%s\.(?:datalad)(?:%s|$)' % (dirsep, dirsep)
+_encoded_dirsep = r'\\'  if on_windows else r'/'
+_VCS_REGEX = r'%s\.(?:git|gitattributes|svn|bzr|hg)(?:%s|$)' % (
+    _encoded_dirsep, _encoded_dirsep)
+_DATALAD_REGEX = r'%s\.(?:datalad)(?:%s|$)' % (
+    _encoded_dirsep, _encoded_dirsep)
 
 
 def find_files(regex, topdir=curdir, exclude=None, exclude_vcs=True, exclude_datalad=False, dirs=False):
@@ -275,7 +313,6 @@ def find_files(regex, topdir=curdir, exclude=None, exclude_vcs=True, exclude_dat
     dirs: bool, optional
       Whether to match directories as well as files
     """
-
     for dirpath, dirnames, filenames in os.walk(topdir):
         names = (dirnames + filenames) if dirs else filenames
         # TODO: might want to uniformize on windows to use '/'
@@ -327,6 +364,40 @@ def is_explicit_path(path):
         or path.startswith(os.pardir + os.sep)
 
 
+# handle this dance once, and import pathlib from here
+# in all other places
+
+from pathlib import (
+    Path,
+    PurePath,
+    PurePosixPath,
+)
+
+if sys.version_info.major == 3 and sys.version_info.minor < 6:
+    # Path.resolve() doesn't have strict=False until 3.6
+    # monkey patch it -- all code imports this class from this
+    # module
+    Path._datalad_moved_resolve = Path.resolve
+
+    def _resolve_without_strict(self, strict=False):
+        if strict or self.exists():
+            # this is pre 3.6 behavior
+            return self._datalad_moved_resolve()
+
+        # if strict==False, find the closest component
+        # that actually exists and resolve that one
+        for p in self.parents:
+            if not p.exists():
+                continue
+            resolved = p._datalad_moved_resolve()
+            # append the rest that did not exist
+            return resolved / self.relative_to(p)
+        # pathlib return the unresolved if nothing resolved
+        return self
+
+    Path.resolve = _resolve_without_strict
+
+
 def rotree(path, ro=True, chmod_files=True):
     """To make tree read-only or writable
 
@@ -403,6 +474,8 @@ def rmdir(path, *args, **kwargs):
 def get_open_files(path, log_open=False):
     """Get open files under a path
 
+    Note: This function is very slow on Windows.
+
     Parameters
     ----------
     path : str
@@ -421,7 +494,9 @@ def get_open_files(path, log_open=False):
     files = {}
     # since the ones returned by psutil would not be aware of symlinks in the
     # path we should also get realpath for path
-    path = realpath(path)
+    # do absolute() in addition to always get an absolute path
+    # even with non-existing paths on windows
+    path = str(Path(path).resolve().absolute())
     for proc in psutil.process_iter():
         try:
             open_paths = [p.path for p in proc.open_files()] + [proc.cwd()]
@@ -506,7 +581,7 @@ def escape_filename(filename):
 def encode_filename(filename):
     """Encode unicode filename
     """
-    if isinstance(filename, text_type):
+    if isinstance(filename, str):
         return filename.encode(sys.getfilesystemencoding())
     else:
         return filename
@@ -519,7 +594,7 @@ def decode_input(s):
     If fails -- issue warning and decode allowing for errors
     being replaced
     """
-    if isinstance(s, text_type):
+    if isinstance(s, str):
         return s
     else:
         encoding = sys.stdin.encoding or 'UTF-8'
@@ -550,19 +625,20 @@ else:
         smtime = time.strftime("%Y%m%d%H%M.%S", time.localtime(mtime))
         lgr.log(3, "Setting mtime for %s to %s == %s", filepath, mtime, smtime)
         Runner().run(['touch', '-h', '-t', '%s' % smtime, filepath])
-        rfilepath = realpath(filepath)
-        if islink(filepath) and exists(rfilepath):
+        filepath = Path(filepath)
+        rfilepath = filepath.resolve()
+        if filepath.is_symlink() and rfilepath.exists():
             # trust noone - adjust also of the target file
             # since it seemed like downloading under OSX (was it using curl?)
             # didn't bother with timestamps
             lgr.log(3, "File is a symlink to %s Setting mtime for it to %s",
                     rfilepath, mtime)
-            os.utime(rfilepath, (time.time(), mtime))
+            os.utime(str(rfilepath), (time.time(), mtime))
         # doesn't work on OSX
         # Runner().run(['touch', '-h', '-d', '@%s' % mtime, filepath])
 
 
-def assure_tuple_or_list(obj):
+def ensure_tuple_or_list(obj):
     """Given an object, wrap into a tuple if not list or tuple
     """
     if isinstance(obj, (list, tuple)):
@@ -570,24 +646,24 @@ def assure_tuple_or_list(obj):
     return (obj,)
 
 
-def assure_iter(s, cls, copy=False, iterate=True):
+def ensure_iter(s, cls, copy=False, iterate=True):
     """Given not a list, would place it into a list. If None - empty list is returned
 
     Parameters
     ----------
     s: list or anything
     cls: class
-      Which iterable class to assure
+      Which iterable class to ensure
     copy: bool, optional
       If correct iterable is passed, it would generate its shallow copy
     iterate: bool, optional
-      If it is not a list, but something iterable (but not a text_type)
+      If it is not a list, but something iterable (but not a str)
       iterate over it.
     """
 
     if isinstance(s, cls):
         return s if not copy else shallow_copy(s)
-    elif isinstance(s, text_type):
+    elif isinstance(s, str):
         return cls((s,))
     elif iterate and hasattr(s, '__iter__'):
         return cls(s)
@@ -597,7 +673,7 @@ def assure_iter(s, cls, copy=False, iterate=True):
         return cls((s,))
 
 
-def assure_list(s, copy=False, iterate=True):
+def ensure_list(s, copy=False, iterate=True):
     """Given not a list, would place it into a list. If None - empty list is returned
 
     Parameters
@@ -606,13 +682,13 @@ def assure_list(s, copy=False, iterate=True):
     copy: bool, optional
       If list is passed, it would generate a shallow copy of the list
     iterate: bool, optional
-      If it is not a list, but something iterable (but not a text_type)
+      If it is not a list, but something iterable (but not a str)
       iterate over it.
     """
-    return assure_iter(s, list, copy=copy, iterate=iterate)
+    return ensure_iter(s, list, copy=copy, iterate=iterate)
 
 
-def assure_list_from_str(s, sep='\n'):
+def ensure_list_from_str(s, sep='\n'):
     """Given a multiline string convert it to a list of return None if empty
 
     Parameters
@@ -628,7 +704,7 @@ def assure_list_from_str(s, sep='\n'):
     return s.split(sep)
 
 
-def assure_dict_from_str(s, **kwargs):
+def ensure_dict_from_str(s, **kwargs):
     """Given a multiline string with key=value items convert it to a dictionary
 
     Parameters
@@ -645,7 +721,7 @@ def assure_dict_from_str(s, **kwargs):
         return s
 
     out = {}
-    for value_str in assure_list_from_str(s, **kwargs):
+    for value_str in ensure_list_from_str(s, **kwargs):
         if '=' not in value_str:
             raise ValueError("{} is not in key=value format".format(repr(value_str)))
         k, v = value_str.split('=', 1)
@@ -656,21 +732,25 @@ def assure_dict_from_str(s, **kwargs):
     return out
 
 
-def assure_bytes(s, encoding='utf-8'):
-    """Convert/encode unicode to str (PY2) or bytes (PY3) if of 'text_type'
+def ensure_bytes(s, encoding='utf-8'):
+    """Convert/encode unicode string to bytes.
+
+    If `s` isn't a string, return it as is.
 
     Parameters
     ----------
     encoding: str, optional
       Encoding to use.  "utf-8" is the default
     """
-    if not isinstance(s, text_type):
+    if not isinstance(s, str):
         return s
     return s.encode(encoding)
 
 
-def assure_unicode(s, encoding=None, confidence=None):
-    """Convert/decode to unicode (PY2) or str (PY3) if of 'binary_type'
+def ensure_unicode(s, encoding=None, confidence=None):
+    """Convert/decode bytestring to unicode.
+
+    If `s` isn't a bytestring, return it as is.
 
     Parameters
     ----------
@@ -681,7 +761,7 @@ def assure_unicode(s, encoding=None, confidence=None):
       A value between 0 and 1, so if guessing of encoding is of lower than
       specified confidence, ValueError is raised
     """
-    if not isinstance(s, binary_type):
+    if not isinstance(s, bytes):
         return s
     if encoding is None:
         # Figure out encoding, defaulting to 'utf-8' which is our common
@@ -713,12 +793,12 @@ def assure_unicode(s, encoding=None, confidence=None):
         return s.decode(encoding)
 
 
-def assure_bool(s):
+def ensure_bool(s):
     """Convert value into boolean following convention for strings
 
     to recognize on,True,yes as True, off,False,no as False
     """
-    if isinstance(s, string_types):
+    if isinstance(s, str):
         if s.isdigit():
             return bool(int(s))
         sl = s.lower()
@@ -735,7 +815,7 @@ def as_unicode(val, cast_types=object):
     """Given an arbitrary value, would try to obtain unicode value of it
     
     For unicode it would return original value, for python2 str or python3
-    bytes it would use assure_unicode, for None - an empty (unicode) string,
+    bytes it would use ensure_unicode, for None - an empty (unicode) string,
     and for any other type (see `cast_types`) - would apply the unicode 
     constructor.  If value is not an instance of `cast_types`, TypeError
     is thrown
@@ -747,12 +827,12 @@ def as_unicode(val, cast_types=object):
     """
     if val is None:
         return u''
-    elif isinstance(val, text_type):
+    elif isinstance(val, str):
         return val
     elif isinstance(val, unicode_srctypes):
-        return assure_unicode(val)
+        return ensure_unicode(val)
     elif isinstance(val, cast_types):
-        return text_type(val)
+        return str(val)
     else:
         raise TypeError(
             "Value %r is not of any of known or provided %s types"
@@ -822,7 +902,7 @@ def map_items(func, v):
     No type checking of values passed to func is done, so `func`
     should be resilient to values which it should not handle
 
-    Initial usecase - apply_recursive(url_fragment, assure_unicode)
+    Initial usecase - apply_recursive(url_fragment, ensure_unicode)
     """
     # map all elements within item
     return v.__class__(
@@ -876,8 +956,8 @@ def generate_file_chunks(files, cmd=None):
     cmd: str or list of str, optional
       Command to account for as well
     """
-    files = assure_list(files)
-    cmd = assure_list(cmd)
+    files = ensure_list(files)
+    cmd = ensure_list(cmd)
 
     maxl = max(map(len, files)) if files else 0
     chunk_size = max(
@@ -958,7 +1038,7 @@ def optional_args(decorator):
         def dec(f):
             return decorator(f, *args, **kwargs)
 
-        is_decorating = not kwargs and len(args) == 1 and isinstance(args[0], collections.Callable)
+        is_decorating = not kwargs and len(args) == 1 and isinstance(args[0], Callable)
         if is_decorating:
             f = args[0]
             args = []
@@ -1011,6 +1091,73 @@ def line_profile(func):
     return newfunc
 
 
+@optional_args
+def collect_method_callstats(func):
+    """Figure out methods which call the method repeatedly on the same instance
+
+    Use case(s):
+      - .repo is expensive since does all kinds of checks.
+      - .config is expensive transitively since it calls .repo each time
+
+
+    TODO:
+    - fancy one could look through the stack for the same id(self) to see if
+      that location is already in memo.  That would hint to the cases where object
+      is not passed into underlying functions, causing them to redo the same work
+      over and over again
+    - ATM might flood with all "1 lines" calls which are not that informative.
+      The underlying possibly suboptimal use might be coming from their callers.
+      It might or not relate to the previous TODO
+    """
+    from collections import defaultdict
+    import traceback
+    from time import time
+    memo = defaultdict(lambda: defaultdict(int))  # it will be a dict of lineno: count
+    # gross timing
+    times = []
+    toppath = op.dirname(__file__) + op.sep
+
+    @wraps(func)
+    def newfunc(*args, **kwargs):
+        try:
+            self = args[0]
+            stack = traceback.extract_stack()
+            caller = stack[-2]
+            stack_sig = \
+                "{relpath}:{s.name}".format(
+                    s=caller, relpath=op.relpath(caller.filename, toppath))
+            sig = (id(self), stack_sig)
+            # we will count based on id(self) + wherefrom
+            memo[sig][caller.lineno] += 1
+            t0 = time()
+            return func(*args, **kwargs)
+        finally:
+            times.append(time() - t0)
+            pass
+
+    def print_stats():
+        print("The cost of property {}:".format(func.__name__))
+        if not memo:
+            print("None since no calls")
+            return
+        # total count
+        counts = {k: sum(v.values()) for k,v in memo.items()}
+        total = sum(counts.values())
+        ids = {self_id for (self_id, _) in memo}
+        print(" Total: {} calls from {} objects with {} contexts taking {:.2f} sec"
+              .format(total, len(ids), len(memo), sum(times)))
+        # now we need to sort by value
+        for (self_id, caller), count in sorted(counts.items(), key=lambda x: x[1], reverse=True):
+            print("  {} {}: {} from {} lines"
+                  .format(self_id, caller, count, len(memo[(self_id, caller)])))
+
+    # Upon total exit we print the stats
+    import atexit
+    atexit.register(print_stats)
+
+    return newfunc
+
+
 # Borrowed from duecredit to wrap duecredit-handling to guarantee failsafe
 def never_fail(f):
     """Assure that function never fails -- all exceptions are caught
@@ -1118,7 +1265,7 @@ def swallow_outputs():
 
     from .ui import ui
     # preserve -- they could have been mocked already
-    oldprint = getattr(__builtin__, 'print')
+    oldprint = getattr(builtins, 'print')
     oldout, olderr = sys.stdout, sys.stderr
     olduiout = ui.out
     adapter = StringIOAdapter()
@@ -1126,12 +1273,12 @@ def swallow_outputs():
     try:
         sys.stdout, sys.stderr = adapter.handles
         ui.out = adapter.handles[0]
-        setattr(__builtin__, 'print', fake_print)
+        setattr(builtins, 'print', fake_print)
 
         yield adapter
     finally:
         sys.stdout, sys.stderr, ui.out = oldout, olderr, olduiout
-        setattr(__builtin__, 'print',  oldprint)
+        setattr(builtins, 'print',  oldprint)
         adapter.cleanup()
 
 
@@ -1257,7 +1404,8 @@ def swallow_logs(new_level=None, file_=None, name='datalad'):
         # TODO: if file_ and there was an exception -- most probably worth logging it?
         # although ideally it should be the next log outside added to that file_ ... oh well
     finally:
-        lgr.handlers, lgr.level = old_handlers, old_level
+        lgr.handlers = old_handlers
+        lgr.setLevel(old_level)
         adapter.cleanup()
 
 
@@ -1325,7 +1473,7 @@ def setup_exceptionhook(ipython=False):
         sys.excepthook = _datalad_pdb_excepthook
 
 
-def assure_dir(*args):
+def ensure_dir(*args):
     """Make sure directory exists.
 
     Joins the list of arguments to an os-specific path to the desired
@@ -1353,7 +1501,7 @@ _pwd_mode = None
 def _switch_to_getcwd(msg, *args):
     global _pwd_mode
     _pwd_mode = 'cwd'
-    lgr.warning(
+    lgr.debug(
         msg + ". From now on will be returning os.getcwd(). Directory"
                " symlinks in the paths will be resolved",
         *args
@@ -1415,7 +1563,9 @@ def getpwd():
                 raise
         try:
             pwd = os.environ['PWD']
-            pwd_real = op.realpath(pwd)
+            # do absolute() in addition to always get an absolute path
+            # even with non-existing paths on windows
+            pwd_real = str(Path(pwd).resolve().absolute())
             # This logic would fail to catch the case where chdir did happen
             # to the directory where current PWD is pointing to, e.g.
             # $> ls -ld $PWD
@@ -1455,6 +1605,8 @@ class chpwd(object):
     def __init__(self, path, mkdir=False, logsuffix=''):
 
         if path:
+            # PY35 has no auto-conversion of Path to str
+            path = str(path)
             pwd = getpwd()
             self._prev_pwd = pwd
         else:
@@ -1625,11 +1777,15 @@ def make_tempfile(content=None, wrapped=None, **tkwargs):
 
     filename = {False: tempfile.mktemp,
                 True: tempfile.mkdtemp}[mkdir](**tkwargs_)
-    filename = realpath(filename)
+    filename = Path(filename).resolve()
 
     if content:
-        with open(filename, 'w' + ('b' if isinstance(content, binary_type) else '')) as f:
-            f.write(content)
+        (filename.write_bytes
+         if isinstance(content, bytes)
+         else filename.write_text)(content)
+
+    # TODO globbing below can also be done with pathlib
+    filename = str(filename)
 
     if __debug__:
         # TODO mkdir
@@ -1685,7 +1841,7 @@ def get_logfilename(dspath, cmd='datalad'):
     """
     assert(exists(dspath))
     assert(isdir(dspath))
-    ds_logdir = assure_dir(dspath, '.git', 'datalad', 'logs')  # TODO: use WEB_META_LOG whenever #789 merged
+    ds_logdir = ensure_dir(dspath, '.git', 'datalad', 'logs')  # TODO: use WEB_META_LOG whenever #789 merged
     return opj(ds_logdir, 'crawl-%s.log' % get_timestamp_suffix())
 
 
@@ -1751,53 +1907,42 @@ def get_dataset_root(path):
     The root path is returned in the same absolute or relative form
     as the input argument. If no associated dataset exists, or the
     input path doesn't exist, None is returned.
-    """
-    suffix = '.git'
-    if not isdir(path):
-        path = dirname(path)
-    apath = abspath(path)
-    # while we can still go up
-    while psplit(apath)[1]:
-        if exists(opj(path, suffix)):
-            return path
-        # new test path in the format we got it
-        path = normpath(opj(path, os.pardir))
-        # no luck, next round
-        apath = abspath(path)
-    return None
 
-
-def get_dataset_pwds(dataset):
-    """Return the current directory for the dataset.
+    If `path` is a symlink or something other than a directory, its
+    the root dataset containing its parent directory will be reported.
+    If none can be found, at a symlink at `path` is pointing to a
+    dataset, `path` itself will be reported as the root.
 
     Parameters
     ----------
-    dataset : Dataset
+    path : Path-like
 
     Returns
     -------
-    A tuple, where the first item is the absolute path of the pwd and the
-    second is the pwd relative to the dataset's path.
+    str or None
     """
-    if dataset:
-        pwd = dataset.path
-        rel_pwd = curdir
-    else:
-        # act on the whole dataset if nothing else was specified
+    path = str(path)
+    suffix = '.git'
+    altered = None
+    if op.islink(path) or not op.isdir(path):
+        altered = path
+        path = op.dirname(path)
+    apath = op.abspath(path)
+    # while we can still go up
+    while op.split(apath)[1]:
+        if op.exists(op.join(path, suffix)):
+            return path
+        # new test path in the format we got it
+        path = op.normpath(op.join(path, os.pardir))
+        # no luck, next round
+        apath = op.abspath(path)
+    # if we applied dirname() at the top, we give it another go with
+    # the actual path, if it was itself a symlink, it could be the
+    # top-level dataset itself
+    if altered and op.exists(op.join(altered, suffix)):
+        return altered
 
-        # Follow our generic semantic that if dataset is specified,
-        # paths are relative to it, if not -- relative to pwd
-        pwd = getpwd()
-        # Pass pwd to get_dataset_root instead of os.path.curdir to handle
-        # repos whose leading paths have a symlinked directory (see the
-        # TMPDIR="/var/tmp/sym link" test case).
-        dataset = get_dataset_root(pwd)
-
-        if dataset:
-            rel_pwd = relpath(pwd, dataset)
-        else:
-            rel_pwd = pwd  # and leave handling to caller
-    return pwd, rel_pwd
+    return None
 
 
 # ATM used in datalad_crawler extension, so do not remove yet
@@ -1840,7 +1985,7 @@ def try_multiple_dec(f, ntrials=None, duration=0.1, exceptions=None, increment_t
             if on_windows else OSError
     if not ntrials:
         # Life goes fast on proper systems, no need to delay it much
-        ntrials = 50 if on_windows else 3
+        ntrials = 100 if on_windows else 10
 
     assert increment_type in {None, 'exponential'}
 
@@ -1909,7 +2054,7 @@ def slash_join(base, extension):
 def safe_print(s):
     """Print with protection against UTF-8 encoding errors"""
     # A little bit of dance to be able to test this code
-    print_f = getattr(__builtin__, "print")
+    print_f = getattr(builtins, "print")
     try:
         print_f(s)
     except UnicodeEncodeError:
@@ -1980,8 +2125,8 @@ def read_csv_lines(fname, dialect=None, readahead=16384, **kwargs):
                 )
                 dialect = 'excel-tab'
 
-    kw = {} if PY2 else dict(encoding='utf-8')
-    with open(fname, 'rb' if PY2 else 'r', **kw) as tsvfile:
+    kw = dict(encoding='utf-8')
+    with open(fname, 'r', **kw) as tsvfile:
         # csv.py doesn't do Unicode; encode temporarily as UTF-8:
         csv_reader = csv.reader(
             tsvfile,
@@ -1991,7 +2136,7 @@ def read_csv_lines(fname, dialect=None, readahead=16384, **kwargs):
         header = None
         for row in csv_reader:
             # decode UTF-8 back to Unicode, cell by cell:
-            row_unicode = map(assure_unicode, row)
+            row_unicode = map(ensure_unicode, row)
             if header is None:
                 header = list(row_unicode)
             else:
@@ -2223,7 +2368,7 @@ def create_tree(path, tree, archives_leading_dir=True, remove_existing=False):
             if full_name.endswith('.gz'):
                 open_func = gzip.open
             with open_func(full_name, "wb") as f:
-                f.write(assure_bytes(load, 'utf-8'))
+                f.write(ensure_bytes(load, 'utf-8'))
         if executable:
             os.chmod(full_name, os.stat(full_name).st_mode | stat.S_IEXEC)
 
@@ -2233,7 +2378,7 @@ def get_suggestions_msg(values, known, sep="\n        "):
     """
     import difflib
     suggestions = []
-    for value in assure_list(values):  # might not want to do it if we change presentation below
+    for value in ensure_list(values):  # might not want to do it if we change presentation below
         suggestions += difflib.get_close_matches(value, known)
     suggestions = unique(suggestions)
     msg = "Did you mean any of these?"
@@ -2247,5 +2392,151 @@ def get_suggestions_msg(values, known, sep="\n        "):
     return ''
 
 
+def bytes2human(n, format='%(value).1f %(symbol)sB'):
+    """
+    Convert n bytes into a human readable string based on format.
+    symbols can be either "customary", "customary_ext", "iec" or "iec_ext",
+    see: http://goo.gl/kTQMs
+
+      >>> from datalad.utils import bytes2human
+      >>> bytes2human(1)
+      '1.0 B'
+      >>> bytes2human(1024)
+      '1.0 KB'
+      >>> bytes2human(1048576)
+      '1.0 MB'
+      >>> bytes2human(1099511627776127398123789121)
+      '909.5 YB'
+
+      >>> bytes2human(10000, "%(value).1f %(symbol)s/sec")
+      '9.8 K/sec'
+
+      >>> # precision can be adjusted by playing with %f operator
+      >>> bytes2human(10000, format="%(value).5f %(symbol)s")
+      '9.76562 K'
+
+    Taken from: http://goo.gl/kTQMs and subsequently simplified
+    Original Author: Giampaolo Rodola' <g.rodola [AT] gmail [DOT] com>
+    License: MIT
+    """
+    n = int(n)
+    if n < 0:
+        raise ValueError("n < 0")
+    symbols = ('', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
+    prefix = {}
+    for i, s in enumerate(symbols[1:]):
+        prefix[s] = 1 << (i + 1) * 10
+    for symbol in reversed(symbols[1:]):
+        if n >= prefix[symbol]:
+            value = float(n) / prefix[symbol]
+            return format % locals()
+    return format % dict(symbol=symbols[0], value=n)
+
+
+def quote_cmdlinearg(arg):
+    """Perform platform-appropriate argument quoting"""
+    # https://stackoverflow.com/a/15262019
+    return '"{}"'.format(
+        arg.replace('"', '""')
+    ) if on_windows else shlex_quote(arg)
+
+
+def split_cmdline(s):
+    """Perform platform-appropriate command line splitting.
+
+    Identical to `shlex.split()` on non-windows platforms.
+
+    Modified from https://stackoverflow.com/a/35900070
+    """
+    if not on_windows:
+        return shlex_split(s)
+
+    # the rest is for windows
+    RE_CMD_LEX = r'''"((?:""|\\["\\]|[^"])*)"?()|(\\\\(?=\\*")|\\")|(&&?|\|\|?|\d?>|[<])|([^\s"&|<>]+)|(\s+)|(.)'''
+
+    args = []
+    accu = None   # collects pieces of one arg
+    for qs, qss, esc, pipe, word, white, fail in re.findall(RE_CMD_LEX, s):
+        if word:
+            pass   # most frequent
+        elif esc:
+            word = esc[1]
+        elif white or pipe:
+            if accu is not None:
+                args.append(accu)
+            if pipe:
+                args.append(pipe)
+            accu = None
+            continue
+        elif fail:
+            raise ValueError("invalid or incomplete shell string")
+        elif qs:
+            word = qs.replace('\\"', '"').replace('\\\\', '\\')
+            if platform == 0:
+                word = word.replace('""', '"')
+        else:
+            word = qss   # may be even empty; must be last
+
+        accu = (accu or '') + word
+
+    if accu is not None:
+        args.append(accu)
+
+    return args
+
+
+def get_wrapped_class(wrapped):
+    """Determine the command class a wrapped __call__ belongs to"""
+    mod = sys.modules[wrapped.__module__]
+    command_class_name = wrapped.__qualname__.split('.')[-2]
+    _func_class = mod.__dict__[command_class_name]
+    lgr.debug("Determined class of decorated function: %s", _func_class)
+    return _func_class
+
+
+# TODO whenever we feel ready for English kill the compat block below
+assure_tuple_or_list = ensure_tuple_or_list
+assure_iter = ensure_iter
+assure_list = ensure_list
+assure_list_from_str = ensure_list_from_str
+assure_dict_from_str = ensure_dict_from_str
+assure_bytes = ensure_bytes
+assure_unicode = ensure_unicode
+assure_bool = ensure_bool
+assure_dir = ensure_dir
+
 
 lgr.log(5, "Done importing datalad.utils")
+
+
+def check_symlink_capability(path, target):
+    """helper similar to datalad.tests.utils.has_symlink_capability
+
+    However, for use in a datalad command context, we shouldn't
+    assume to be able to write to tmpfile and also not import a whole lot from
+    datalad's test machinery. Finally, we want to know, whether we can create a
+    symlink at a specific location, not just somewhere. Therefore use
+    arbitrary path to test-build a symlink and delete afterwards. Suiteable
+    location can therefore be determined by high lever code.
+
+    Parameters
+    ----------
+    path: Path
+    target: Path
+
+    Returns
+    -------
+    bool
+    """
+
+    try:
+        target.touch()
+        path.symlink_to(target)
+        return True
+    except Exception:
+        return False
+    finally:
+        if path.exists():
+            path.unlink()
+        if target.exists():
+            target.unlink()
